<?php
/*
Plugin Name: <a href="http://reciprocity.be/ctc">Configurable Tag Cloud</a>
Description: A tag cloud widget for Wordpress 2.3 to allow the changing of style options from the Widgets control panel.  Now with color!
Author: Keith Solomon
Version: 2.51
Author URI: http://reciprocity.be/
*/

// Colorizes tag links based on weight
// If nothing gets passed in to the function for $mincolor and/or $maxcolor, it defaults to black (#000000).
function ColorWeight($weight, $mincolor, $maxcolor) {
	if ($weight) {
		$weight = $weight/100;

		if (strlen($mincolor) == 4) {
			$r = substr($mincolor, 1, 1);
			$g = substr($mincolor, 2, 1);
			$b = substr($mincolor, 3, 1);

			$mincolor = "#$r$r$g$g$b$b";
		}

		if (strlen($maxcolor) == 4) {
			$r = substr($maxcolor, 1, 1);
			$g = substr($maxcolor, 2, 1);
			$b = substr($maxcolor, 3, 1);

			$maxcolor = "#$r$r$g$g$b$b";
		}

		$minr = hexdec(substr($mincolor, 1, 2));
		$ming = hexdec(substr($mincolor, 3, 2));
		$minb = hexdec(substr($mincolor, 5, 2));

		$maxr = hexdec(substr($maxcolor, 1, 2));
		$maxg = hexdec(substr($maxcolor, 3, 2));
		$maxb = hexdec(substr($maxcolor, 5, 2));

		$r = dechex(intval((($maxr - $minr) * $weight) + $minr));
		$g = dechex(intval((($maxg - $ming) * $weight) + $ming));
		$b = dechex(intval((($maxb - $minb) * $weight) + $minb));

		if (strlen($r) == 1) $r = "0" . $r;
		if (strlen($g) == 1) $g = "0" . $g;
		if (strlen($b) == 1) $b = "0" . $b;

		$color = "#$r$g$b";
		$color = substr($color,0,7);
		
		return $color;
	}
}

// My tag cloud function - modified to allow changing the colors of the tags based on weight
function tag_cloud( $args = '' ) {
	$defaults = array(
		'smallest' => 8, 'largest' => 22, 'unit' => 'pt', 'number' => 45,
		'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC',
		'exclude' => '', 'include' => '', 'mincolor' => '', 'maxcolor' => ''
	);
	$args = wp_parse_args( $args, $defaults );

	$tags = get_tags( array_merge($args, array('orderby' => 'count', 'order' => 'DESC')) ); // Always query top tags

	if ( empty($tags) )
		return;

	$return = generate_tag_cloud( $tags, $args ); // Here's where those top tags get sorted according to $args
	if ( is_wp_error( $return ) )
		return false;
	else 
		echo apply_filters( 'tag_cloud', $return, $args );
}

// $tags = prefetched tag array ( get_tags() )
// $args['format'] = 'flat' => whitespace separated, 'list' => UL, 'array' => array()
// $args['orderby'] = 'name', 'count'
function generate_tag_cloud( $tags, $args = '' ) {
	global $wp_rewrite;
	$defaults = array(
		'smallest' => 8, 'largest' => 22, 'unit' => 'pt', 'number' => 45,
		'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC',
		'exclude' => '', 'include' => '', 'mincolor' => '', 'maxcolor' => ''
	);
	$args = wp_parse_args( $args, $defaults );

	extract($args);

	if ( !$tags )
		return;
	$counts = $tag_links = array();
	foreach ( (array) $tags as $tag ) {
		$counts[$tag->name] = $tag->count;
		$tag_links[$tag->name] = get_tag_link( $tag->term_id );
		if ( is_wp_error( $tag_links[$tag->name] ) )
			return $tag_links[$tag->name];
		$tag_ids[$tag->name] = $tag->term_id;
	}

	$min_count = min($counts);
	$spread = max($counts) - $min_count;
	if ( $spread <= 0 )
		$spread = 1;
	$font_spread = $largest - $smallest;
	if ( $font_spread <= 0 )
		$font_spread = 1;
	$font_step = $font_spread / $spread;

	// SQL cannot save you; this is a second (potentially different) sort on a subset of data.
	if ( 'name' == $orderby )
		uksort($counts, 'strnatcasecmp');
	else
		asort($counts);

	if ( 'DESC' == $order )
		$counts = array_reverse( $counts, true );

	$a = array();

	$rel = ( is_object($wp_rewrite) && $wp_rewrite->using_permalinks() ) ? ' rel="tag"' : '';

	foreach ( $counts as $tag => $count ) {
		if ( $largest == $smallest )
			$tag_weight = $largest;
		else
			$tag_weight = ($smallest+(($count-$min_count)*$font_step));
		$diff = $largest-$smallest;
		if ( $diff <= 0 )
			$diff = 1;
		$color_weight = round(99*($tag_weight-$smallest)/($diff)+1);
		$tag_id = $tag_ids[$tag];
		$tag_link = clean_url($tag_links[$tag]);
		$tag = str_replace(' ', '&nbsp;', wp_specialchars( $tag ));
		$a[] = "<a href='$tag_link' class='tag-link-$tag_id' title='" . attribute_escape( sprintf( __('%d topics'), $count ) ) . "'$rel style='font-size: " . $tag_weight
			. "$unit; color: " .
				ColorWeight($color_weight, $mincolor, $maxcolor)
			. "'>$tag</a>";
	}

	switch ( $format ) :
	case 'array' :
		$return =& $a;
		break;
	case 'list' :
		$return = "<ul class='wp-tag-cloud'>\n\t<li>";
		$return .= join("</li>\n\t<li>", $a);
		$return .= "</li>\n</ul>\n";
		break;
	default :
		$return = join("\n", $a);
		break;
	endswitch;

	return apply_filters( 'generate_tag_cloud', $return, $tags, $args );
}

// This gets called at the plugins_loaded action
function widget_ctc_init() {
	
	// Check for the required API functions
	if ( !function_exists('register_sidebar_widget') || !function_exists('register_widget_control') )
		return;

	// This saves options and prints the widget's config form.
	function widget_ctc_control() {
		$options = $newoptions = get_option('widget_ctc');
		if ( $_POST['ctc-submit'] ) {
			$newoptions['title'] = strip_tags(stripslashes($_POST['ctc-title']));
			$newoptions['number'] = strip_tags(stripslashes($_POST['ctc-number']));
			$newoptions['unit'] = strip_tags(stripslashes($_POST['ctc-unit']));
			$newoptions['smallest'] = strip_tags(stripslashes($_POST['ctc-smallest']));
			$newoptions['largest'] = strip_tags(stripslashes($_POST['ctc-largest']));
			$newoptions['mincolor'] = strip_tags(stripslashes($_POST['ctc-mincolor']));
			$newoptions['maxcolor'] = strip_tags(stripslashes($_POST['ctc-maxcolor']));
			$newoptions['format'] = strip_tags(stripslashes($_POST['ctc-format']));
			$newoptions['orderby'] = strip_tags(stripslashes($_POST['ctc-orderby']));
			$newoptions['order'] = strip_tags(stripslashes($_POST['ctc-order']));
			//$newoptions['tags'] = explode(' ', trim(strip_tags(stripslashes($_POST['ctc-tags']))));
		}
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_ctc', $options);
		}
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$number = htmlspecialchars($options['number'], ENT_QUOTES);
		$unit = htmlspecialchars($options['unit'], ENT_QUOTES);
		$smallest = htmlspecialchars($options['smallest'], ENT_QUOTES);
		$largest = htmlspecialchars($options['largest'], ENT_QUOTES);
		$mincolor = htmlspecialchars($options['mincolor'], ENT_QUOTES);
		$maxcolor = htmlspecialchars($options['maxcolor'], ENT_QUOTES);
		$format = htmlspecialchars($options['format'], ENT_QUOTES);
		$orderby = htmlspecialchars($options['orderby'], ENT_QUOTES);
		$order = htmlspecialchars($options['order'], ENT_QUOTES);
?>
	
	<div style="text-align:center">
		<h3>Configurable Tag Cloud Options</h3>
		<span style="line-height:15px"><br /><br /></span>
		<table>
			<tr><td><strong>Title</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-title" name="ctc-title" value="<?php echo wp_specialchars($options['title'], true); ?>" /></td>
				<td style="font-size:0.75em">Title shown in sidebar.</td>
			</tr>
			<tr><td><strong>Number of Tags to Display</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-number" name="ctc-number" value="<?php echo wp_specialchars($options['number'], true); ?>" /></td>
				<td style="font-size:0.75em">Controls the total number of tags in your cloud.</td>
			</tr>
			<tr><td><strong>Font Display Unit</strong></td>
				<td style="text-align:right;">
					<select id="ctc-unit" name="ctc-unit" size="1" value="" />
			   			<option value="px" <?=($unit=="px")?'selected':''?>>Pixel</option>
			   			<option value="pt" <?=($unit=="pt")?'selected':''?>>Point</option>
			   			<option value="em" <?=($unit=="em")?'selected':''?>>Em</option>
				   		<option value="%" <?=($unit=="%")?'selected':''?>>Percent</option>
		   			</select>
				</td>
				<td style="font-size:0.75em">What unit to use for font sizes.</td>
			</tr>
			<tr><td><strong>Smallest Font Size</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-smallest" name="ctc-smallest" value="<?php echo wp_specialchars($options['smallest'], true); ?>" /></td>
				<td style="font-size:0.75em">Tags will be displayed no smaller than this value.</td>
			</tr>
			<tr><td><strong>Largest Font Size</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-largest" name="ctc-largest" value="<?php echo wp_specialchars($options['largest'], true); ?>" /></td>
				<td style="font-size:0.75em">Tags will be displayed no larger that this value.</td>
			</tr>
			<tr><td><strong>Min. Tag Color</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-mincolor" name="ctc-mincolor" value="<?php echo wp_specialchars($options['mincolor'], true); ?>" /></td>
				<td style="font-size:0.75em">Beginning color for tag gradient.  Please include the #.</td>
			</tr>
			<tr><td><strong>Max. Tag Color</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-maxcolor" name="ctc-maxcolor" value="<?php echo wp_specialchars($options['maxcolor'], true); ?>" /></td>
				<td style="font-size:0.75em">Ending color for tag gradient.  Please include the #.</td>
			</tr>
			<tr><td><strong>Cloud Format</strong></td>
				<td style="text-align:right;">
					<select id="ctc-format" name="ctc-format" size="1" value="" />
				   		<option value="flat" <?=($format=="flat")?'selected':''?>>Flat</option>
				   		<option value="list" <?=($format=="list")?'selected':''?>>List</option>
						<!-- Add back if you really need it... -->
				   		<!-- <option value="array" <?=($format=="array")?'selected':''?>>Array</option> -->
		   			</select>
				</td>
				<td style="font-size:0.75em">How to display the cloud.</td>
			</tr>
			<tr><td><strong>Sort By</strong></td>
				<td><input type="radio" id="ctc-orderby" name="ctc-orderby" <?=($orderby=="name")?'checked="checked"':''?> value="name" /> Name <input type="radio" id="ctc-orderby" name="ctc-orderby" <?=($orderby=="count")?'checked="checked"':''?> value="count" /> Post Count</td>
				<td style="font-size:0.75em">What field to sort by.</td>
			</tr>
			<tr><td><strong>Sort Order</strong></td>
				<td style="font-size:0.85em"><input type="radio" id="ctc-order" name="ctc-order" <?=($order=="ASC")?'checked="checked"':''?> value="ASC" /> Ascending <input type="radio" id="ctc-order" name="ctc-order" <?=($order=="DESC")?'checked="checked"':''?> value="DESC" /> Descending</td>
				<td style="font-size:0.75em">Direction of sort.</td>
			</tr>
			<!-- <tr><td><strong>Tags To Exclude</strong></td>
				<td>
					<input style="text-align:right" type="text" id="ctc-tags" name="ctc-tags" value="<?php echo wp_specialchars(implode(' ', (array) $options['tags']), true); ?>" />
				</td>
				<td style="font-size:0.75em">Optional</td>
			</tr> -->
		</table>
		<input type="hidden" name="ctc-submit" id="ctc-submit" value="1" />
	</div>

<?php
	}

	// The widget itself
	function widget_ctc($args) {
	extract($args);
	$defaults = array('title' => 'Tags', 'number' => '45', 'unit' => 'pt', 'smallest' => 8, 'largest' => 22, 'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC', 'mincolor' => '', 'maxcolor' => '');
	$options = (array) get_option('widget_ctc');

	foreach ( $defaults as $key => $value )
		if ( $options[$key] == "" )
			$options[$key] = $defaults[$key];

		$tagcloud = 'smallest='.$options['smallest'];
		$tagcloud.= '&largest='.$options['largest'];
		$tagcloud.= '&mincolor='.$options['mincolor'];
		$tagcloud.= '&maxcolor='.$options['maxcolor'];
		$tagcloud.= '&unit='.$options['unit'];
		$tagcloud.= '&format='.$options['format'];
		$tagcloud.= '&number='.$options['number'];
		$tagcloud.= '&orderby='.$options['orderby'];
		$tagcloud.= '&order='.$options['order'];
		//$tagcloud.= count($options['tags']) ? '&exclude='.implode(',', $options['tags']) : '';
		
		//echo 'tag cloud string: '.$tagcloud.'<br />';

	echo $before_widget;
	echo $before_title . $options['title'] . $after_title;
	echo '<div class="ctc">';
		tag_cloud($tagcloud);
	echo '</div>';
	echo $after_widget;
}

	// Tell Dynamic Sidebar about our new widget and its control
	register_sidebar_widget(array('CTC', 'widgets'), 'widget_ctc');
	register_widget_control(array('CTC', 'widgets'), 'widget_ctc_control', 420, 410);
	
}

// Delay plugin execution to ensure Dynamic Sidebar has a chance to load first
add_action('widgets_init', 'widget_ctc_init');

?>